#!/usr/bin/env ruby
#
# Ruby script for preparing libkdcraw releases from KDE SVN
#
# (c) 2006 Angelo Naselli <anaselli at linux dot it>
# 
# License: GNU General Public License V2

name       = "libkdcraw"
egmodule   = "libs"
# tarball version
version    = "0.1.1"
# KDCRAW_VERSION into version.h (has to be equal to version)
version_n  = "0x000101"
# Makefile.am libtool version (current:revision:age)
version_info = "1:0:0"
# Makefile.am api_version
ABI_MAJOR = "1"

usesv2cl   = "yes"
# svn2cl depends on how you installed it
svn2cl     = "svn2cl"
svnbase    = "svn+ssh://anaselli@svn.kde.org/home/kde"
#svnbase    = "svn+ssh://vfuoglio@svn.kde.org/home/kde"
#svnroot    = "trunk/extragear"
svnroot    = "branches/extragear/kde3"


# last release revision tag + 1 (look at released Changelog) 
chlog_rev  = "670713"

t          = Time.now
date       = t.strftime("%Y-%m-%d")

# modify libkdcraw.lsm
old_file   = name + "/" + name + ".lsm"
new_file   = name + "/" + name + ".lsm.new"

old = File.open(old_file)
new = File.open(new_file, "w")
while old.gets do
    fileLine = $_
    if (fileLine.index("Version") )
      new.print  "Version: " + version + "\n"
    elsif (fileLine.index("Entered-date") )
      new.print  "Entered-date: " + date + "\n"
    else
      new.print  fileLine
    end
end
old.close
new.close
File.rename(old_file, "old.orig")
File.rename(new_file, old_file)
puts( "done #{old_file}\n" )

# modify libkdcraw/libkdcraw/version.h
old_file   = name + "/" + name + "/version.h"
new_file   = name + "/" + name + "/version.h.new"

old = File.open(old_file)
new = File.open(new_file, "w")
while old.gets do
    fileLine = $_
    if (fileLine.index("kipi_version") )
      new.print  "static const char kipi_version[] = \"" + version + "\";\n"
    elsif (fileLine.index("KIPI_VERSION") )
       if (fileLine.index("0x"))
         new.print  "#define KIPI_VERSION " + version_n +"\n"
       else
         new.print  fileLine
       end
    else
      new.print  fileLine
    end
end
old.close
new.close
File.rename(old_file, "old.orig")
File.rename(new_file, old_file)
puts( "done #{old_file}\n" )

# modify libkdcraw/libkdcraw.pc.in
old_file   = name + "/" + name + ".pc.in"
new_file   = name + "/" + name + ".pc.in.new"

old = File.open(old_file)
new = File.open(new_file, "w")
while old.gets do
    fileLine = $_
    if (fileLine.index("Version") )
      new.print  "Version: " + version + "\n"
    else
      new.print  fileLine
    end
end
old.close
new.close
File.rename(old_file, "old.orig")
File.rename(new_file, old_file)
puts( "done #{old_file}\n" )

# modify libkdcraw/libkdcraw/Makefile.am
old_file   = name + "/" + name + "/" + "Makefile.am"
new_file   = name + "/" + name + "/" + "Makefile.am.new"

old = File.open(old_file)
new = File.open(new_file, "w")
while old.gets do
    fileLine = $_
    if (fileLine.index("libkdcraw_version_info =") )
      new.print  "libkdcraw_version_info = " + version_info + "\n"
    else
      new.print  fileLine
    end
end
old.close
new.close
File.rename(old_file, "old.orig")
File.rename(new_file, old_file)
puts( "done #{old_file}\n" )

# modify libkdcraw/configure.in.in
old_file   = name + "/configure.in.in"
new_file   = name + "/configure.in.in.new"

old = File.open(old_file)
new = File.open(new_file, "w")
while old.gets do
    fileLine = $_
    if (fileLine.index("ABI_MAJOR=") )
      new.print  "ABI_MAJOR=" + ABI_MAJOR + "\n"
    elsif (fileLine.index("VARSION_INFO=") )
      new.print  "VARSION_INFO= \"" + version_info + "\"\n"
    else
      new.print  fileLine
    end
end
old.close
new.close
File.rename(old_file, "old.orig")
File.rename(new_file, old_file)
puts( "done #{old_file}\n" )

# modify libkdcraw/Changelog
if (usesv2cl  == "yes")
	chfile = name + "/ChangeLog"
	`rm -f #{chfile} 2> /dev/null`
	chlog = File.open(chfile, "w")
	chlog.print "V " + version + " - " + date + "\n"
	chlog.print "----------------------------------------------------------------------------"
	chlog.print "\n"
	chlog.close
	
`#{svn2cl} --break-before-msg \
    --strip-prefix=#{svnroot}/#{egmodule} --stdout \
    -i #{svnbase}/#{svnroot}/#{egmodule}/#{name} -r head:#{chlog_rev} >> #{chfile}`

	`echo \"----------------------------------------------------------------------------\" >> #{chfile}`
	`svn cat #{svnbase}/#{svnroot}/#{egmodule}/#{name}/ChangeLog >> #{chfile}`
	puts( "done #{chfile}\n" )
	puts( "\n" )
end



